package com.gx.obe.config.utils;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import com.gx.obe.config.ConfigActivator;

/**
 * 用于插件项目和非插件项目，提供两者通用的方法接口
 */
public class ProjectUtils {
//	private static AbstractUIPlugin plugin = CommonActivator.getDefault();

	private ProjectUtils() {
	}

	public static String ROOT_PATH = getRoor(ConfigActivator.getDefault());

	/**
	 * 系统的当前路径
	 **/
	public static String getRoor(AbstractUIPlugin plugin){
		if(null == ROOT_PATH || "".equals(ROOT_PATH)){
			
			String root = ProjectUtils.toFullPath(plugin, "/");
			// 如果前面有/，读取文件时找不到
			if(root.charAt(0)=='/'){
				root = root.substring(1);
			}
			ROOT_PATH = root;
		}
		return ROOT_PATH;
	}
	
	/**
	 * @Description: 获得系统相对目录下的绝对路径
	 * @author guoyr
	 * @param filePath
	 * @return
	 */
	public static String getFileAbsolutePath(AbstractUIPlugin plugin, String filePath){
		try {
			return FileLocator.toFileURL(
					ProjectUtils.getURL(plugin, filePath)).getPath();
		} catch (IOException e) {
			return null;
		}
	}


	public static URL getURL(AbstractUIPlugin plugin, String path) {

		URL url = null;
		if (null != plugin) {// 如果是插件
			try {
				url = FileLocator.find(plugin.getBundle(), new Path(path), null);
			} catch (Exception e) {
				LogUtils.logError(path + "  is not found!!", e);
			}
			return url;
		}
		else
			try {
				return new URL("file:" + path);
			} catch (MalformedURLException e) {
				throw new RuntimeException(path + " is error", e);
			}
	
	}

	public static InputStream getInputStream(AbstractUIPlugin plugin, String path) {
		URL url = getURL(plugin, path);
		try {
			return url.openStream();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	public static String [] toFullPath(AbstractUIPlugin plugin, String[] path){
		int size = path.length;
		String [] paths = new String [size];
		List<String> fullpaths = new ArrayList <String>();
		for(int i=0;i<size;i++){
			paths[i]=toFullPath(plugin, path[i]);
		}
		return paths;
	}

	public static String toFullPath(AbstractUIPlugin plugin, String path) {
		if (null != plugin) {
			try {
				return FileLocator.toFileURL(ProjectUtils.getURL(plugin, path))
						.getPath();
			} catch (IOException e) {
				throw new RuntimeException(path + " toFullPath is fault", e);
			}
		} else {
			return path;
		}
	}
	
	/**
	 * @Description: 获得文件的流字节
	 * @author guoyr
	 * @param filePath 相对路径 如：icons/sign.png
	 * @return
	 */
	public static byte[] getFileInputStreamByte(AbstractUIPlugin plugin, String filePath){
		FileInputStream in = null;
		byte[] imageByte = null;
		try {
			in = new FileInputStream(ProjectUtils.getFileAbsolutePath(plugin, filePath));
			imageByte = InputStreamUtil.inputStreamTOByte(in);
		} catch (FileNotFoundException e) {
		}finally{
			if(null != in){
				try {
					in.close();
				} catch (IOException e) {
				}
			}
		}
		return imageByte;
	}
}
