package com.gx.obe.config.utils;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.FileLocator;

import com.gx.obe.config.ConfigActivator;
 
/** 
  * @Description: PropertiesUtils加载配置文件            		
  * @author guoyr
  */ 
public class PropertiesUtils {
	private static Map<String, String> enumMap = new HashMap<String, String>();
	private static Map<String, String> propertyMap = new HashMap<String, String>();
	private static Map<String, String> webserviceMap = new HashMap<String, String>();
	private static Properties enumProperties;
	private static Properties properties;
	private static Properties webserviceProperties;
	private static Logger LOG = LogUtils.logger;

	/**
	 * @Description: 获得枚举
	 * @author guoyr
	 * @param enumName
	 * @return
	 */
	public static String getEnum(String enumName){
		return getEnum(enumName, "");
	}
	
	/**
	 * @Description: 获得枚举
	 * @author guoyr
	 * @param enumName
	 * @param defaultEnumValue 默认值
	 * @return
	 */
	public static String getEnum(String enumName, String defaultEnumValue){
		String enumValue = enumMap.get(enumName);
		if(hasLength(enumValue)){
			return enumValue;
		}
		if(null == enumProperties){
			enumProperties = getProperties("enum.properties");
		}
		if(null != enumProperties){
			enumValue = enumProperties.getProperty(enumName,defaultEnumValue);
			enumMap.put(enumName, enumValue);
		}
		return enumValue;
	}
	
	/**
	 * @Description: 设置配置文件的值
	 * @author guoyr
	 * @param propertyName
	 * @param value
	 */
	public static void setEnumValue(String propertyName, String value){
		enumMap.put(propertyName, value);
		enumProperties.put(propertyName, value);
	}
	
	/**
	 * @Description: 保存配置文件
	 * @author guoyr
	 * @param propertyName
	 * @param value 
	 */
	public static void saveEnum(String propertyName, String value){
		if(null == enumProperties){
			enumProperties = getProperties("enum.properties");
		}
		if(null != enumProperties){
			String propValue = enumProperties.getProperty(propertyName);
			if(null == propValue){
				propValue = "";
			}
			if(null == value){
				value = "";
			}
			if(!propValue.equals(value)){
				setEnumValue(propertyName, value);
				saveProperties(enumProperties);
			}
		}
	}
	
	/**
	 * @Description: 获得webservice配置文件的属性
	 * @author guoyr
	 * @param propertyName
	 * @param defaultpropertyValue
	 * @return
	 */
	public static String getWebserviceProperty(String propertyName, String defaultpropertyValue){
		String propertyValue = webserviceMap.get(propertyName);
		if(hasLength(propertyValue)){
			return propertyValue;
		}
		if(null == webserviceProperties){
			if(null == webserviceProperties){
				webserviceProperties = getProperties("webservice.properties");
			}
		}
		
		if(null != webserviceProperties){
			propertyValue = webserviceProperties.getProperty(propertyName,defaultpropertyValue);
			webserviceMap.put(propertyName, propertyValue);
		}
		return propertyValue;
	}
	
	/**
	 * @Description: 设置配置文件的值
	 * @author guoyr
	 * @param propertyName
	 * @param value
	 */
	public static void setPropertyValue(String propertyName, String value){
		propertyMap.put(propertyName, value);
		properties.put(propertyName, value);
	}

	/**
	 * @Description: 保存配置文件
	 * @author guoyr
	 * @param propertyName
	 * @param value 
	 */
	public static void saveProperty(String propertyName, String value){
		if(null == properties){
			properties = getProperties("config.properties");
		}
		if(null != properties){
			String propValue = properties.getProperty(propertyName);
			if(null == propValue){
				propValue = "";
			}
			if(null == value){
				value = "";
			}
			if(!propValue.equals(value)){
				setPropertyValue(propertyName, value);
				saveProperties(properties);
			}
		}
	}
	/**
	 * @Description: 获得配置文件的属性
	 * @author guoyr
	 * @param propertyName
	 * @param defaultpropertyValue
	 * @return
	 */
	public static String getProperty(String propertyName, String defaultpropertyValue){
		String propertyValue = propertyMap.get(propertyName);
		if(hasLength(propertyValue)){
			return propertyValue;
		}
		if(null == properties){
			properties = getProperties("config.properties");
		}
		if(null != properties){
			propertyValue = properties.getProperty(propertyName);
			if(null == propertyValue || propertyValue.length() == 0){
				propertyValue = defaultpropertyValue;
			}
			propertyMap.put(propertyName, propertyValue);
		}
		return propertyValue;
	}
	
	/**
	 * @Description: 获得配置文件的属性
	 * @author guoyr
	 * @param propertyName
	 * @param defaultpropertyValue
	 * @return
	 */
	public static String getProperty(String propertyName){
		return getProperty(propertyName, null);
	}
	
	public static void reLoadProperties(){
		properties = getProperties("config.properties");
		propertyMap.clear();
	}
	
	/**
	 * @Description: 获得Properties
	 * @author guoyr
	 * @param propertieName
	 * @return
	 */
	public static Properties getProperties(String propertieName){
		Properties properties = new Properties();
		try {
			if(null != ConfigActivator.getDefault()){
				String propertsPath = FileLocator.toFileURL(ProjectUtils.getURL(ConfigActivator.getDefault(), propertieName)).getPath();
				InputStream in = new FileInputStream(propertsPath);
				properties.load(in);
				properties.setProperty("properties_path", propertsPath);
			}
		} catch (IOException e) {
			LOG.error(e.getMessage(), e);
		}
		return properties;
	}
	
	/**
	 * @Description: 获得Properties对象
	 * @author guoyr
	 * @param source
	 * @return
	 */
	public static Properties loadProperties(String propertiesFilePath){
		Properties properties = new Properties();
		InputStream in = null;
		try {
			in = new FileInputStream(propertiesFilePath);
			properties.load(in);
		} catch (FileNotFoundException e) {
			LOG.error(propertiesFilePath, e);
		} catch (IOException e) {
			LOG.error(propertiesFilePath, e);
		}finally{
			if(null != in){
				try {
					in.close();
					in = null;
				} catch (IOException e) {
					LOG.error(propertiesFilePath, e);
				}
			}
		}
		return properties;
	}
	
	
	/**
	 * @Description: 保存Properties
	 * @author guoyr
	 * @param properties
	 */
	public static void saveProperties(Properties properties, boolean...readLoadProperties){
		if(null != properties && isNotEmpty(properties.getProperty("properties_path"))){
			FileOutputStream fos;
			try {
				fos = new FileOutputStream(properties.getProperty("properties_path"));
				properties.store(fos, "update");
				fos.close();
			} catch (Exception e) {
				LOG.error("", e);
			}finally{
				boolean clean = true;
				if(null == readLoadProperties || readLoadProperties.length == 0){
					clean = true;
				}else {
					clean = readLoadProperties[0];
				}
				if(clean){
					enumMap.clear();
					propertyMap.clear();
					webserviceMap.clear();
				}
			}
		}
	}

	/**
	 * @Description: 保存配置文件
	 * @author guoyr
	 * @param properties
	 * @param filePath 
	 */
	public static boolean saveProperties(Properties properties, String filePath) {
		// 保存配置文件数据
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(filePath);
			properties.store(fos, "update");
			fos.flush();
			return true;
		} catch (Exception e) {
			LOG.error(filePath, e);
		}finally{
			if(null != fos){
				try {
					fos.close();
					fos = null;
				} catch (IOException e) {
					LOG.error(filePath, e);
				}
			}
		}
		return false;
	}
	/**
	 * 根据配置文件方言名修改方言值
	 * @param fileName
	 * @param propertyValueName
	 * @param propertyValue
	 * @return
	 */
	public static void setPropertieValue(String fileName,String propertyValueName, String propertyValue) {
		if(propertyValue==null){
			propertyValue="";
		}
		Properties properties = new Properties();
		try {
			String filePath = FileLocator.toFileURL(ProjectUtils.getURL(ConfigActivator.getDefault(), fileName)).getPath();
			InputStream in = new FileInputStream(filePath);
			properties.load(in);
			in.close();

			properties.setProperty(propertyValueName, propertyValue);
			FileOutputStream fos = new FileOutputStream(filePath);
			properties.store(fos, "update");
			fos.close();
		
		} catch (IOException e) {
			LOG.error(e.getMessage(), e);
		}
		
	}
	
	/**
	 * 将资源文件内容加载在内存中
	 * @param filePath
	 * @return
	 */
	public static Map<String, String> readProperties(String fileName) {
		Map<String, String> map = new HashMap<String, String>();
		Properties props = new Properties();
		try {
//			InputStream ips = new BufferedInputStream(new FileInputStream(filePath));
			InputStream ips = new BufferedInputStream(new FileInputStream(FileLocator.toFileURL(
					ProjectUtils.getURL(ConfigActivator.getDefault(), fileName)).getPath()));
			props.load(ips);
			Enumeration<?> enum1 = props.propertyNames();
			while(enum1.hasMoreElements()){
				String key = (String)enum1.nextElement();
				String value = props.getProperty(key);
				System.out.println(key+"="+value);
				map.put(key, value);
			}
		} catch (FileNotFoundException e) {
			LOG.error(e.getMessage(), e);
		} catch (IOException e) {
			LOG.error(e.getMessage(), e);
		}
		return map;
	}

	public static void copyAllProperties(String sourcePropertiesPath, String targetPropertiesPath){

		// 获得源配置文件(*.properties)数据
		Properties sourcePropFile = loadProperties(sourcePropertiesPath);
		Properties targetPropFile = loadProperties(targetPropertiesPath);
		if(null == sourcePropFile || null == targetPropFile){
			return;
		}
		Enumeration<?> sourceEnum = sourcePropFile.propertyNames();
		if(null != sourceEnum){
			// 获得目标配置文件(*.properties)对象
			while(sourceEnum.hasMoreElements()){
				String sourceKey = (String) sourceEnum.nextElement();
				targetPropFile.setProperty(sourceKey, getObjString(sourcePropFile.get(sourceKey)));
			}
			// 保存配置文件数据
			FileOutputStream fos = null;
			try {
				fos = new FileOutputStream(targetPropertiesPath);
				targetPropFile.store(fos, "update");
				fos.flush();
			} catch (Exception e) {
				LOG.error(sourcePropertiesPath, e);
			}finally{
				if(null != fos){
					try {
						fos.close();
						fos = null;
					} catch (IOException e) {
						LOG.error(sourcePropertiesPath, e);
					}
				}
			}
		}
	
	}
	public static void copyPropertiesFile(String sourcePropertiesPath, String targetPropertiesPath, String[] copyProperties){
		// 获得源配置文件(*.properties)数据
		Properties sourcePropFile = loadProperties(sourcePropertiesPath);
		Properties targetPropFile = loadProperties(targetPropertiesPath);
		if(null == sourcePropFile || null == targetPropFile){
			return;
		}
		Enumeration<?> sourceEnum = sourcePropFile.propertyNames();
		if(null != sourceEnum && null != copyProperties && copyProperties.length > 0){
			List<String> copyPropertiesList = Arrays.asList(copyProperties);
			// 获得目标配置文件(*.properties)对象
			while(sourceEnum.hasMoreElements()){
				String sourceKey = (String) sourceEnum.nextElement();
				if(targetPropFile.containsKey(sourceKey)){
					// 复制升级前的配置属性
					if(copyPropertiesList.contains(sourceKey)){
						targetPropFile.setProperty(sourceKey, getObjString(sourcePropFile.get(sourceKey)));
					}
				}else {
					targetPropFile.setProperty(sourceKey, getObjString(sourcePropFile.get(sourceKey)));
				}
			}
			// 保存配置文件数据
			FileOutputStream fos = null;
			try {
				fos = new FileOutputStream(targetPropertiesPath);
				targetPropFile.store(fos, "update");
				fos.flush();
			} catch (Exception e) {
				LOG.error(sourcePropertiesPath, e);
			}finally{
				if(null != fos){
					try {
						fos.close();
						fos = null;
					} catch (IOException e) {
						LOG.error(sourcePropertiesPath, e);
					}
				}
			}
		}
	}
	
	private static String getObjString(Object obj){
		if(null == obj){
			return "";
		}else {
			return obj.toString();
		}
	}
	
	public static boolean hasLength(CharSequence str) {
		return (str != null && str.length() > 0);
	}

	
	public static boolean isNotEmpty(String input) {
		return null != input && input.length() > 0;
	}
	
	public static void main(String[] args) {
		PropertiesUtils.readProperties("F:/workspace_work/evaluate/config.properties");
	}
	
}
